#include <iostream>
#include "3dengfx/3dengfx.hpp"
#include "common/timer.h"
#include "fxwt/fxwt.hpp"
#include "3dengfx/psys.hpp"

#include "SDL/SDL.h"
#include <unistd.h>

#include "common/image.h"

bool Init();
void CleanUp();
void UpdateGfx();
void KeyHandler(int key);

Scene scene;
ntimer timer;
ParticleSystem *p;
TargetCamera *cam;
bool shot = false;

int main() {
	if(!Init()) {
		return -1;
	}

	return fxwt::MainLoop();
}

bool Init() {
	try {
		GraphicsInitParameters gip = LoadGraphicsContextConfig("3dengfx.conf");
		CreateGraphicsContext(gip);
	}
	catch(EngineException e) {
		std::cerr << e.GetMessage() << std::endl;
		return false;
	}

	fxwt::SetDisplayHandler(UpdateGfx);
	fxwt::SetIdleHandler(UpdateGfx);
	fxwt::SetKeyboardHandler(KeyHandler);
	atexit(CleanUp);

	Light *light = new PointLight(Vector3(50, 100, -100));
	scene.AddLight(light);

	cam = new TargetCamera(Vector3(0, 9, -32), Vector3(0, 5, 0));
	scene.AddCamera(cam);
	
	Object *plane = new Object;
	CreatePlane(plane->GetTriMeshPtr(), Plane(Vector3(0, 0, 0)), Vector2(500, 500), 10);
	plane->Rotate(Vector3(-half_pi, 0, 0));
	plane->GetMaterialPtr()->diffuse_color = Color(0.5f, 0.5f, 0.5f);

	//scene.AddObject(plane);

	p = new ParticleSystem("test.psys");
	p->SetPosition(Vector3(0, 10.0, 0));
	
	MotionController cntl(CTRL_SIN, TIME_WRAP);
	cntl.SetSinFunc(2.0, 8.0);
	cntl.SetControlAxis(CTRL_X);

	p->AddController(cntl, CTRL_TRANSLATION);

	cntl.SetControlAxis(CTRL_Z);
	cntl.SetControllerType(CTRL_COS);
	p->AddController(cntl, CTRL_TRANSLATION);

	cntl.SetControlAxis(CTRL_Y);
	cntl.SetSinFunc(1.0, 5.0);
	p->AddController(cntl, CTRL_TRANSLATION);


	timer_reset(&timer);
	return true;
}

void UpdateGfx() {
	unsigned long msec = timer_getmsec(&timer);
	float t = msec / 1000.0f;

	psys::SetGlobalTime(msec);
	p->Update();
	
	Clear(0);
	ClearZBufferStencil(1.0f, 0);

	scene.Render();
	p->Draw();
	
	usleep(33000);
	
	Flip();
}

void CleanUp() {
	delete p;
	DestroyGraphicsContext();
}

void KeyHandler(int key) {
	static int cursor_hidden = 1;
	switch(key) {
	case 'a':
		cam->Zoom(0.8);
		break;

	case 'z':
		cam->Zoom(1.2);
		break;

	case '`':
		shot = true;
		ScreenCapture();
		break;
	
	case 'h':
		cursor_hidden = (cursor_hidden + 1) % 2;
		SDL_ShowCursor(cursor_hidden);
		break;
		
	case 27:
		exit(0);

	default:
		break;
	}
}
